use lazy_static::lazy_static;
use spin::Mutex;
use uart_16550::SerialPort;

use crate::os::util::without_interrupts;

lazy_static! {
	pub static ref SERIAL1: Mutex<SerialPort> = {
		let mut serial_port = unsafe { SerialPort::new(0x3F8) };
		serial_port.init();
		Mutex::new(serial_port)
	};
}

#[doc(hidden)]
pub fn _print(args: core::fmt::Arguments<'_>) {
	use core::fmt::Write;

	// Disable interrupts to prevent deadlocks
	// (we might get an interrupt while printing)
	without_interrupts(|| {
		SERIAL1
			.lock()
			.write_fmt(args)
			.expect("Printing to serial failed");
	})
}

/// Prints to the host through the serial interface.
#[macro_export]
macro_rules! print {
	($($arg:tt)*) => {
		$crate::drivers::serial::_print(format_args!($($arg)*));
	};
}

/// Prints to the host through the serial interface, appending a newline.
#[macro_export]
macro_rules! println {
	() => ($crate::print!("\n"));
	($fmt:expr) => ($crate::print!(concat!($fmt, "\n")));
	($fmt:expr, $($arg:tt)*) => ($crate::print!(
		concat!($fmt, "\n"), $($arg)*));
}
